@@ -20,7 +20,7 @@ from . import dataclasses as dc
 from .exceptions import HTTPError
 
 # Metadata
-__version__ = "2.1.0"
+__version__ = "2.2.1"
 __author__ = "Clappform B.V."
 __email__ = "info@clappform.com"
 __license__ = "MIT"
@@ -60,7 +60,7 @@ class Clappform:
 
     _auth: dc.Auth = None
 
-    def __init__(self, base_url: str, username: str, password: str):
+    def __init__(self, base_url: str, username: str, password: str, timeout: int = 2):
         self._base_url: str = f"{base_url}/api"
 
         #: Username to use in the :meth:`auth <auth>`
@@ -69,12 +69,15 @@ class Clappform:
         #: Password to use in the :meth:`auth <auth>`
         self.password: str = password
 
+        #: HTTP request timeout in seconds.
+        self.timeout: int = timeout
+
     def _default_user_agent(self) -> str:
         """Return a string with version of requests and clappform packages."""
         requests_ua = r.utils.default_user_agent()
         return f"clappform/{__version__} {requests_ua}"
 
-    def _request(self, method: str, path: str, timeout=10, **kwargs):
+    def _request(self, method: str, path: str, **kwargs):
         """Implements :class:`requests.request`."""
         headers = kwargs.pop("headers", {})
         headers["User-Agent"] = self._default_user_agent()
@@ -82,7 +85,7 @@ class Clappform:
             method,
             f"{self._base_url}{path}",
             headers=headers,
-            timeout=timeout,
+            timeout=self.timeout,
             **kwargs,
         )
         doc = resp.json()
@@ -144,6 +147,9 @@ class Clappform:
         document = self._private_request("GET", "/version")
         return dc.Version(**document["data"])
 
+    def _remove_nones(self, original: dict) -> dict:
+        return {k: v for k, v in original.items() if v is not None}
+
     def _app_path(self, app) -> str:
         if isinstance(app, dc.App):
             return app.path()
@@ -251,7 +257,8 @@ class Clappform:
         """
         if not isinstance(app, dc.App):
             raise TypeError(f"app arg is not of type {dc.App}, got {type(app)}")
-        document = self._private_request("PUT", app.path(), json=asdict(app))
+        payload = self._remove_nones(asdict(app))
+        document = self._private_request("PUT", app.path(), json=payload)
         return dc.App(**document["data"])
 
     def delete_app(self, app) -> dc.ApiResponse:
@@ -380,7 +387,9 @@ class Clappform:
         )
         return dc.Collection(**document["data"])
 
-    def create_collection(self, app, slug: str, name: str, desc: str) -> dc.Collection:
+    def create_collection(
+        self, app, slug: str, name: str, desc: str, db: str = "MONGO"
+    ) -> dc.Collection:
         """Create a new Collection.
 
         :param app: App identifier to create collection for.
@@ -388,6 +397,8 @@ class Clappform:
         :param str slug: Name used for internal identification.
         :param str name: Name of the collection.
         :param str desc: Description of what data the collection holds.
+        :param str db: Database where collection is stored. Valid values for ``db`` are
+            ``MONGO`` and ``DATALAKE``, defaults to: ``MONGO``
 
         Usage::
 
@@ -410,6 +421,9 @@ class Clappform:
         """
         path = self._app_path(app)
         path = path.replace("/app/", "/collection/")
+        valid_databases = ("MONGO", "DATALAKE")
+        if db not in valid_databases:
+            raise ValueError(f"db kwarg value is not one of: {valid_databases}")
         document = self._private_request(
             "POST",
             path,
@@ -417,6 +431,7 @@ class Clappform:
                 "slug": slug,
                 "name": name,
                 "description": desc,
+                "database": db,
             },
         )
         return dc.Collection(**document["data"])
@@ -439,7 +454,7 @@ class Clappform:
             >>> collection.name = "Spam & Eggs Collection"
             >>> collection = c.update_collection(collection)
 
-        :riased TypeError: collection arg is not of type
+        :raises TypeError: collection arg is not of type
             :class:`clappform.dataclasses.Collection`
 
         :returns: Updated Collection object
@@ -448,9 +463,8 @@ class Clappform:
         if not isinstance(collection, dc.Collection):
             t = type(collection)
             raise TypeError(f"collection arg is not of type {dc.Collection}, got {t}")
-        document = self._private_request(
-            "PUT", collection.path(), json=asdict(collection)
-        )
+        payload = self._remove_nones(asdict(collection))
+        document = self._private_request("PUT", collection.path(), json=payload)
         return dc.Collection(**document["data"])
 
     def delete_collection(self, collection: dc.Collection) -> dc.ApiResponse:
@@ -598,7 +612,8 @@ class Clappform:
         """
         if not isinstance(query, dc.Query):
             raise TypeError(f"query arg must be of type {dc.Query}, got {type(query)}")
-        document = self._private_request("PUT", query.path(), json=asdict(query))
+        payload = self._remove_nones(asdict(query))
+        document = self._private_request("PUT", query.path(), json=payload)
         return dc.Query(**document["data"])
 
     def delete_query(self, query) -> dc.ApiResponse:
@@ -614,7 +629,7 @@ class Clappform:
         document = self._private_request("DELETE", path)
         return dc.ApiResponse(**document)
 
-    def aggregate_dataframe(self, options: dict, interval_timeout=0.1):
+    def aggregate_dataframe(self, options: dict, interval_timeout: int = 0.1):
         """Aggregate a dataframe
 
         :param dict options: Options for dataframe aggregation.
@@ -671,11 +686,14 @@ class Clappform:
             time.sleep(interval_timeout)  # Prevent Denial Of Service (dos) flagging.
             document = self._private_request(**params)
 
-    def read_dataframe(self, query, limit=100, interval_timeout=0.1):
+    def read_dataframe(self, query, limit: int = 100, interval_timeout: int = 0.1):
         """Read a dataframe.
 
-        :param query: Query to use on collection
-        :type query: clappform.dataclasses.Collection
+        :param query: Query to for retreiving data. When Query is of type
+            :class:`clappform.dataclasses.Collection` everything inside the collection
+            is retreived.
+        :type query: :class:`clappform.dataclasses.Query` |
+            :class:`clappform.dataclasses.Collection`
         :param int limit: Amount of records to retreive per request.
         :param interval_timeout: Time to sleep per request.
         :type interval_timeout: int
@@ -696,14 +714,23 @@ class Clappform:
         :returns: Generator to read dataframe
         :rtype: :class:`generator`
         """
-        if not isinstance(query, dc.Query):
-            raise TypeError(f"query arg mgust be of type {dc.Query}, got {type(query)}")
         path = "/dataframe/read_data"
         params = {
             "method": "POST",
             "path": path,
-            "json": {"query": query.slug, "limit": limit},
+            "json": {"limit": limit},
         }
+        if isinstance(query, dc.Query):
+            params["json"]["query"] = query.slug
+        elif isinstance(query, dc.Collection):
+            params["json"]["app"] = query.app
+            params["json"]["collection"] = query.slug
+        else:
+            t = type(query)
+            raise TypeError(
+                f"query arg must be of type {dc.Query} or {dc.Collection}, got {t}"
+            )
+
         document = self._private_request(**params)
         if "total" not in document or document["total"] == 0:
             return
@@ -844,9 +871,8 @@ class Clappform:
             raise TypeError(
                 f"actionflow arg is not of type {dc.Actionflow}, got {type(actionflow)}"
             )
-        document = self._private_request(
-            "PUT", actionflow.path(), json=asdict(actionflow)
-        )
+        payload = self._remove_nones(asdict(actionflow))
+        document = self._private_request("PUT", actionflow.path(), json=payload)
         return dc.Actionflow(**document["data"])
 
     def delete_actionflow(self, actionflow) -> dc.ApiResponse:
@@ -861,3 +887,113 @@ class Clappform:
         path = self._actionflow_path(actionflow)
         document = self._private_request("DELETE", path)
         return dc.ApiResponse(**document)
+
+    def _questionnaire_path(self, questionnaire) -> str:
+        if isinstance(questionnaire, dc.Questionnaire):
+            return questionnaire.path()
+        if isinstance(questionnaire, int):
+            return dc.Questionnaire._path.format(questionnaire)
+        t = type(questionnaire)
+        raise TypeError(
+            f"questionnaire arg is not of type {dc.Questionnaire} or {int}, got {t}"
+        )
+
+    def get_questionnaires(self, extended: bool = False) -> list[dc.Questionnaire]:
+        """Get all questionnaires
+
+        :param bool extended: Optional retreive fully expanded questionnaires, defaults
+            to ``false``.
+
+        :returns: List of :class:`clappform.dataclasses.Questionnaire` or empty list if
+            there are no questionnaires.
+        :rtype: list[clappform.dataclasses.Questionnaire]
+        """
+        if not isinstance(extended, bool):
+            raise TypeError(
+                f"extended kwarg mut be of type {bool}, got {type(extended)}"
+            )
+        extended = str(extended).lower()
+        document = self._private_request("GET", f"/questionnaires?extended={extended}")
+        return [dc.Questionnaire(**obj) for obj in document["data"]]
+
+    def get_questionnaire(
+        self, questionnaire, extended: bool = False
+    ) -> dc.Questionnaire:
+        """Get a questionnaire
+
+        :param bool extended: Optional retreive fully expanded questionnaire, defaults
+            to ``false``.
+
+        :returns: Qustionnaire Object
+        :rtype: clappform.dataclasses.Questionnaire
+        """
+        if not isinstance(extended, bool):
+            t = type(extended)
+            raise TypeError(f"extended kwarg mut be of type {bool}, got {t}")
+        extended = str(extended).lower()
+        path = self._questionnaire_path(questionnaire)
+        document = self._private_request("GET", f"{path}?extended={extended}")
+        return dc.Questionnaire(**document["data"])
+
+    def create_questionnaire(self, name: str, settings: dict) -> dc.ApiResponse:
+        """Create a new questionnaire.
+
+        :param str name: Display name for the new questionnaire.
+        :param dict settings: Settings object
+
+        :returns: ApiResponse object
+        :rtype: clappform.dataclasses.ApiResponse
+        """
+        document = self._private_request(
+            "POST",
+            "/questionnaire",
+            json={
+                "name": name,
+                "settings": settings,
+            },
+        )
+        return dc.ApiResponse(**document)
+
+    def update_questionnaire(
+        self, questionnaire: dc.Questionnaire, settings: dict
+    ) -> dc.Questionnaire:
+        """Update an existing Questionnaire.
+
+        :param questionnaire: Questionnaire object to update.
+        :type questionnaire: clappform.dataclasses.Questionnaire
+        :param dict settings: Settings object
+
+        :returns: Updated Questionnaire object
+        :rtype: clappform.dataclasses.Questionnaire
+        """
+        if not isinstance(questionnaire, dc.Questionnaire):
+            t = type(questionnaire)
+            raise TypeError(
+                f"questionnaire arg must be of type {dc.Questionnaire}, got {t}"
+            )
+        payload = self._remove_nones(
+            {
+                "active": questionnaire.active,
+                "settings": settings,
+            }
+        )
+        document = self._private_request(
+            "PUT",
+            questionnaire.path(),
+            json=payload,
+        )
+        return dc.Questionnaire(**document["data"])
+
+    def delete_questionnaire(self, questionnaire):
+        """Delete a Questionnaire.
+
+        :param questionnaire: Questionnaire identifier
+        :type questionnaire: :class:`int` |
+            :class:`clappform.dataclasses.Questionnaire`
+
+        :returns: API response object
+        :rtype: clappform.dataclasses.ApiResponse
+        """
+        path = self._questionnaire_path(questionnaire)
+        document = self._private_request("DELETE", path)
+        return dc.ApiResponse(**document)
